
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

# == Schema Information
#
# Table name: users
#
#  id                     :integer          not null, primary key
#  email                  :string           default(""), not null
#  encrypted_password     :string           default(""), not null
#  reset_password_token   :string
#  reset_password_sent_at :datetime
#  remember_created_at    :datetime
#  sign_in_count          :integer          default("0"), not null
#  current_sign_in_at     :datetime
#  last_sign_in_at        :datetime
#  current_sign_in_ip     :string
#  last_sign_in_ip        :string
#  created_at             :datetime         not null
#  updated_at             :datetime         not null
#  provider               :string           default("email"), not null
#  uid                    :string           default(""), not null
#  confirmation_token     :string
#  confirmed_at           :datetime
#  confirmation_sent_at   :datetime
#  unconfirmed_email      :string
#  tokens                 :text
#  name                   :string
#  avatar_file_name       :string
#  avatar_content_type    :string
#  avatar_file_size       :integer
#  avatar_updated_at      :datetime
#  bookmarks_count        :integer          default("0")
#  user_category_id       :integer
#  score                  :float            default("0.0")
#  follows_count          :integer          default("0")
#  description            :text
#  likes_count            :integer
#  learning_objects_count :integer
#  collections_count      :integer
#  nickname               :string
#  cover_file_name        :string
#  cover_content_type     :string
#  cover_file_size        :integer
#  cover_updated_at       :datetime
#  education              :string
#  submitter_request      :integer          default("0")
#  dspace_handle          :string
#  dspace_sets            :text             default("{}")
#  teacher_id             :string
#  birthday               :datetime
#  school_id              :integer
#  terms_accepted_at      :datetime
#  cpf                    :string
#  state                  :integer          default("0")
#  suspended_at           :datetime
#  reactivated_at         :datetime
#  times_blocked          :integer
#  ask_teacher_question   :boolean          default("true")
#  dspace_collection      :string           default("{}")
#  allow_experience_change:boolean
#  experience             :integer          default("0")
#  level                  :integer          default("1")
#  points                 :integer          default("0")
#  last_sign_in_day       :datetime

class User < ApplicationRecord
  include Followable
  include Reputationable
  include Subjectable
  include Tagger
  include Complainable
  include Publisher
  include Trackable

  # Include default devise modules. Others available are:
  # :confirmable, :lockable, :timeoutable and :omniauthable
  devise :database_authenticatable, :registerable,
         :recoverable, :rememberable, :trackable, :validatable,
         :omniauthable

  include DeviseTokenAuth::Concerns::User

  belongs_to :school, optional: true

  has_and_belongs_to_many :roles
  has_and_belongs_to_many :institutions

  has_many :bookmarks
  has_many :bookmarks, as: :bookmarkable
  has_many :bookmark_collections, through: :bookmarks, source: :bookmarkable, source_type: 'Collection'
  has_many :bookmark_learning_objects, through: :bookmarks, source: :bookmarkable, source_type: 'LearningObject'
  #has_many :collections, as: :owner
  #has_many :learning_objects, as: :publisher
  has_many :views
  has_many :downloads
  has_many :likes
  has_many :shares
  has_many :follows
  has_many :reviews
  has_many :applications
  has_many :searches

  has_many :submissions, class_name: "Submission", foreign_key: "submitter_id"
  has_many :curations, class_name: "Submission", foreign_key: "curator_id"
  has_many :curator_assignments
  has_many :assignments, through: :curator_assignments, source: :submission

  has_many  :user_items
  has_many  :items, through: :user_items

  has_many  :progresses
  has_many  :requirements, through: :progresses

  has_many  :unlocked_achievements
  has_many  :achievements, through: :unlocked_achievements

  has_many  :action_counters
  has_many  :actions, through: :action_counters

  after_create :default_role
  before_save :verify_teacher_id
  after_save :verify_dspace_info
  before_destroy :delete_index
  after_update :create_actions

  has_attached_file :avatar, styles: { medium: '300x300>', thumb: '60x60>' }, default_url: ''
  validates_attachment_content_type :avatar, content_type: ['image/jpg', 'image/jpeg', 'image/png', 'image/gif']

  has_attached_file :cover, styles: { medium: '800x300>', thumb: '160x60>' }, default_url: ''
  validates_attachment_content_type :cover, content_type: ['image/jpg', 'image/jpeg', 'image/png', 'image/gif']

  validates :terms_of_service, acceptance: true

  validates :dspace_url, presence: true, if: Proc.new {|u| !u.dspace_handle.blank? || !u.dspace_sets.blank?}

  validates :dspace_handle, presence: true, if: Proc.new {|u| !u.dspace_url.blank? || !u.dspace_sets.blank?}

  enum submitter_request: { default: 0, requested: 1, accepted: 2, rejected: 3 }

  validates :school_id, presence: true, unless: Proc.new {|u| u.default? || u.rejected?}

  enum state: { active: 0, blocked: 1, banished: 2}

  searchkick language: 'brazilian', match: :word_start, searchable: [:name], callbacks: :async, index_prefix: Socket.gethostname.downcase

  acts_as_paranoid

  # don't versionate frequently changed fields
  has_paper_trail ignore: [:tokens, :sign_in_count, :current_sign_in_at, :last_sign_in_at]

  def search_data
    {
      name: name,
      score: score
    }
  end

  def name=(name)
    write_attribute(:name, name.strip) if !name.blank?
  end

  def should_index?
      deleted_at.nil?
  end

  def delete_index
    User.searchkick_index.remove(self)
  end

  def is_admin?
    roles.each do |role|
      return true if role.name == 'admin'
    end
    false
  end

  def is_moderator?
    roles.each do |role|
      return true if role.name == 'moderator'
    end
    false
  end

  def is_curator?
    roles.each do |role|
      return true if role.name == 'curator'
    end
    false
  end

  def is_supervisor?
    roles.each do |role|
      return true if role.name == 'supervisor'
    end
    false
  end

  def is_editor?
    roles.each do |role|
      return true if role.name == 'editor'
    end
    false
  end

  def is_submitter?
    roles.each do |role|
      return true if role.name == 'submitter'
    end
    false
  end

  def is_publisher?
    roles.each do |role|
      return true if role.name == 'publisher'
    end
    false
  end

  def is_partner?
    roles.each do |role|
      return true if role.name == 'partner'
    end
    false
  end

  def can_edit?
    self.is_admin? || self.is_editor?
  end

  def active_for_authentication?
    super && self.active?
  end

  def associated_collections
    c = collections.blank? ? [] : collections.to_a
    institutions.each { |i| c.push(*i.collections.to_a) }
    c
  end

  def associated_learning_objects
    lo = learning_objects.blank? ? [] : learning_objects.to_a
    institutions.each { |i| lo.push(*i.learning_objects.to_a) }
    lo
  end

  def own_reviews
    Review.where(user_id: id)
  end

  def token_validation_response
    UserDeviseSerializer.new(self, root: false)
  end


  def received_reviews
    Review.joins('INNER JOIN learning_objects ON ' \
                  "learning_objects.publisher_id = #{id} AND " \
                  'reviews.reviewable_id = learning_objects.id')
  end

  # ~~~~ gamification actions ~~~~ #

  def add_user_item(item)
    UserItem.where(user_id: self.id, item_id: item.id).first_or_create
  end

  def equip_item(item)
    user_item = add_user_item(item)
    equipped_items = user_items.includes(:item).where(items: {item_type: item.item_type}, being_used: :true)

    if((item.item_type == "badge" && equipped_items.count == 3) || (item.item_type != "badge" && equipped_items.count > 0))
      unequip_item(equipped_items.first)
    end
    user_item.being_used = true
    user_item.save
  end

  def unequip_item(item)
    user_item = user_items.where(item_id: item.id).first
    user_item.being_used = false
    user_item.save
  end

  def remove_item(item)
    UserItem.where(user_id: self.id, item_id: item.id).destroy_all
  end

  def xp_to_level(lvl)
    # necessary xp to reach level x from level x-1
    (100 + Math.log([lvl, 900].min, 1.07)).floor
  end

  def xp_to_next_level
    # necessary xp for the user to reach next level
    xp_to_level(level + 1) - experience
  end

  def calculate_level(xp)
    necessary_xp = xp_to_level(level + 1)
    new_level = self.level
    while xp >= necessary_xp
      new_level += 1
      xp -= necessary_xp
      necessary_xp = xp_to_level(level + 1)
    end
    return new_level, xp
  end

  def experience=(xp)
    new_level, remaining_xp = calculate_level(xp)
    self.level = new_level
    super(remaining_xp)
  end

  def get_achievement(achievement)
    return false if !achievement.active?

    unlocked_achievements << UnlockedAchievement.create(user_id: self.id, achievement_id: achievement.id)

    achievement.items.each do |item|
      add_user_item(item)
    end
    
    self.points += achievement.reward_points
    self.experience += achievement.reward_experience
    self.save

    progresses.includes(requirement: :achievements).where(achievements:{ id: achievement.id }).each do |progress|
      if achievement.repeatable != "never"
        progress.update(counter: 0)
      end
    end
  end

  def complete_action(action, quantity=1)
    return false, { "error": "action not found"} if action.blank?

    action_counter = action_counters.where(action: action).first_or_create
    action_counter.counter += quantity

    if !action_counter.save
      return false, action_counter.errors
    else
      self.update(experience: experience + (action.reward_experience*quantity))
      action.requirements.each do |requirement|
        progress = progresses.where(requirement: requirement).first_or_create
        progress.update(counter: progress.counter + quantity)
        requirement.achievements.each do |achievement|
          if progress.counter >= requirement.goal && unlocked_achievements.where(achievement: achievement).blank?
            incomplete = false
            achievement_progresses = progresses.includes(requirement: :achievements).where(achievements:{ id: achievement.id })
            achievement_progresses.each do |progress|
              incomplete = true if progress.counter < progress.requirement.goal
            end
            progress.update(counter: progress.counter - requirement.goal) if requirement.repeatable != "never"
            get_achievement(achievement) if !incomplete
          end
        end
      end
      return true, action_counter
    end
  end

  def create_actions
    if saved_changes.include?("current_sign_in_at")
      months_since_creation = ((Time.now - created_at)/1.month).to_i
      month_action = Action.find_by_name("Meses de Conta")
      month_counter = action_counters.where(action: month_action).first_or_create.counter

      complete_action(month_action, months_since_creation - month_counter) if months_since_creation >= 1 && months_since_creation > month_counter

      complete_action(Action.find_by_name("Fazer Login")) if current_sign_in_at.to_date != last_sign_in_day.to_date

      update(last_sign_in_day: current_sign_in_at) if current_sign_in_at.to_date > last_sign_in_day.to_date

    elsif saved_changes.include?("avatar")
      complete_action(Action.find_by_name("Adicionar Foto de Perfil"))
    elsif saved_changes.include?("cover")
      complete_action(Action.find_by_name("Adicionar Capa de Perfil"))
    elsif saved_changes.include?("description")
      complete_action(Action.find_by_name("Adicionar Descrição do Usuário"))
    end
  end

  # ~~~~ end of gamification actions ~~~~ #

  # ~~~~ followable actions ~~~~ #
  # An user can follow anothers users and collections
  # Examples:
  # current_user.follow(User.find_by_email('john@revelator.com'))
  # current_user.follow(Collection.find(143))
  def follow(followable)
    Follow.create(user: self, followable: followable)
  end

  # Examples:
  # current_user.unfollow(User.find_by_email('john@revelator.com'))
  # current_user.unfollow(Collection.find(143))
  def unfollow(followable)
    Follow.where(user: self, followable: followable).destroy_all
  end

  # An user can see if is following another user
  # Examples:
  # current_user.following?(User.find_by_email('john@revelator.com')) => true
  # current_user.following?(Collection.find(143)) => true
  def following?(followable)
    !follows.where(followable_id: followable.id, followable_type: followable.class.name).blank?
  end

  # This function permits see what person is following.
  # For current user, list all users and all collections. But,
  # if, isn't current user, list only users, and public collections which this person follows.
  def following(followable_type, is_current_user = true)
    f = Follow.where(user_id: id, followable_type: followable_type).all

    # If type collection, and isn't current_user, so should list only public collections
    if followable_type.to_s == 'Collection' && !is_current_user
      f = Follow.joins('INNER JOIN collections ON follows.followable_id = collections.id').where(
        "follows.user_id = #{id} " \
        'AND follows.followable_type = \'Collection\' ' \
        'AND collections.id = follows.followable_id ' \
        'AND collections.privacy = \'public\''
      )
    end

    f
  end

  # Returns everything that this user have followed or created
  def watching
    watching_array = []
    [User, Collection].each do |model|
      following(model.to_s).each do |follow|
        watching_array.push(*model.where(id: follow.followable_id))
      end
    end
    watching_array.push(*collections)
    watching_array.push(*learning_objects)
    watching_array.push(*reviews)
    watching_array.push(*submissions)
    watching_array.push(*curator_assignments)
  end

  # ~~~~ end followable actions ~~~~ #

  def default_role
    roles << Role.submitter
  end

  def self.from_omniauth(access_token)
    user = User.find_by(email: user_email)
    return nil if user.blank?

    data = access_token.info
    user_email = access_token.provider == 'twitter' ? "#{access_token.uid}@twitter.com" : data['email']
    User.create(
      name: data['name'],
      email: user_email,
      provider: access_token.provider,
      uid: access_token.uid,
      password: Devise.friendly_token[0, 20]
    )
  end

  def verify_dspace_info
    if roles.include?(Role.find_by_name("partner")) && !(changed & ["dspace_url", "dspace_handle", "dspace_sets"]).empty?
      DspaceMailer.dspace_info_updated(self).deliver_now
    end
  end

  def verify_teacher_id
    return true if (changed & ["teacher_id", "birthday"]).empty?

    params = {
      filter: "teacher:#{teacher_id},day: #{birthday.day},month:#{birthday.month},year:#{birthday.year},min_year:2017"
    }
    uri = URI("https://simcaq.c3sl.ufpr.br/api/v1/verify_teacher")
    uri.query = URI.encode_www_form(params)
    response = JSON.parse(Net::HTTP.get(uri))
    if !response['result'].blank?
      roles << Role.publisher
      return true
    end

    errors.add(:INEP, "Dados do INEP incorretos")
    return false
  end

  def activity_owner
    self
  end

  def ignore_changes
    super + %w(tokens sign_in_count current_sign_in_at last_sign_in_at current_sign_in_ip last_sign_in_ip score follows_count confirmation_token confirmed_at confirmation_sent_at)
  end

  def update_tracked_fields(request)
    super
    # Change request.remote_ip to request.env["HTTP_X_REAL_IP"] in production
    self.current_sign_in_ip = request.remote_ip
  end

  def treat_complaintment
  end

  def complaint_accept(option)
    if option == "blocked"
      self.suspended_at = Time.current
      self.reactivated_at = self.suspended_at + 7.day
      self.increment(:times_blocked, 1)
      self.blocked!
    elsif option == "banished"
      self.suspended_at = Time.current
      self.reactivated_at = nil
      self.increment(:times_blocked, 1)
      self.banished!
    elsif
      raise ArgumentError.new(I18n.t("error.wrong_arguments", param: "option"))
    end
  end

  def complaint_reject(params)
  end
end
