
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'openssl'

namespace :import do
  desc 'import learning objects from dspace'

  task :sync_learning_objects => :environment do
    desc 'Synchronize Learning Objects with DSpace Items'
    OpenSSL::SSL::VERIFY_PEER = OpenSSL::SSL::VERIFY_NONE

    params = {
      fq: "dc.date.accessioned_dt:[NOW-1DAY TO NOW]",
      start: 0,
      rows: 100,
      wt: "json",
      indent: "true"
    }
    uri = URI(DspaceService.link + "/solr/search/select")

    users = User.joins(:roles).where(roles: {name: "partner"}).where.not(dspace_url: [nil, ""], dspace_collection: nil)
    users.each do |user|
      p "User: #{user.id}, #{user.name}, #{user.email}"
      next if user.dspace_collection.blank?

      user_params = params
      user_col_str = user.dspace_collection.map{|col| "location.coll:#{col}"}.join(" OR ")
      user_params[:q] = "search.resourcetype:2 AND (#{user_col_str})"

      n_items = 0
      loop do
        begin
          items = []
          uri.query = URI.encode_www_form(user_params)
          response = JSON.parse(Net::HTTP.get(uri))
          if n_items == 0
            n_items = response["response"]["numFound"]
            p "#{n_items} items found"
          end

          break if params[:start] > n_items
          p "--> importing items from #{params[:start]} to #{params[:start] + params[:rows]}"

          response["response"]["docs"].each do |doc|
            next if item_exists?(doc["search.resourceid"])
            item = dspace_client.items.find(
              id: doc["search.resourceid"],
              expand: "metadata,bitstreams,parentCollection"
            )
            if items.is_a? String
              p "Received a string instead of item: #{item}"
              next
            end
            items << item
          end
          p "#{items.count} items retrieved"
          importer = Dspace::LearningObjectImporter.new(items, user, dspace_client)
          importer.import
        rescue PG::ConnectionBad, Faraday::TimeoutError, Net::HTTP::Persistent::Error, Net::ReadTimeout => e
          p "Database error, going to sleep"
          p e
          p e.class
          p e.message
          ActiveRecord::Base.clear_active_connections!
          # Sleeps for a while to wait database's recovery
          sleep(60.seconds)
          # Goes to next iteration to retry
          next
        else
          params[:start] += params[:rows]
        end
      end
    end
  end

  task :learning_object, [:log] => [:environment] do |_t, args|
    desc 'Importing Learning Objects from Dspace items'

    args.with_defaults(log: STDOUT)
    logger = Log::DatabaseLogger.new(args.log)
    logger.level = Log::DatabaseLogger::CREATE

    # Quantity of items fetched on each iteration
    limit = 500

    # Start point from where items will be fetched
    offset = 0

    loop do
      logger.info " --> importing items from #{offset} to #{offset + limit}"

      begin
        # Get items from dspace (from offset to offset+limit)
        dspace_items = get_col_items_from_dspace(limit, offset, ['metadata', 'bitstreams', 'parentCollection'])
      rescue => e
        logger.warn 'Database error, going to sleep'
        logger.error e
        # Sleeps for a while to wait database's recovery
        sleep(10.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more items to import
        break if dspace_items.empty?

        # continue if receive an error string
        if dspace_items.is_a? String
          logger.warn "Received a string instead of items: #{dspace_items}"
          next
        end

        # Increment offset, to get new items on next iteration
        offset += limit

        dspace_items.select! do |i|
          i.parent_collection.id == DspaceService::BD_INT_OBJETOS_COLLECTION
        end
        next if dspace_items.empty?
        # Iterates through items
        # Verifies if item is already on repository
        # Initializes new LearningObjects
        # and saves them on LearningObjects's repository
        importer = Dspace::LearningObjectImporter.new(dspace_items)
        importer.import
      end
    end
    logger.close
  end

  task :update_dspace_ids => :environment do
    items_not_found = 0
    bitstreams_not_found = 0
    loop do
      begin
        n_los = LearningObject.with_deleted.where("id_dspace ~ '^[0-9]+$'").count
        n_atts = LearningObject::Attachment.where("id_dspace ~ '^[0-9]+$'").count
        p "LearningObjects with legacy id_dspace = #{n_los}"
        p "Items not found = #{items_not_found}"
        p "Attachments with legacy id_dspace = #{n_atts}"
        p "Bitstreams not found = #{bitstreams_not_found}"
        if n_los > items_not_found
          LearningObject.with_deleted.where("id_dspace ~ '^[0-9]+$'").find_each do |lo|
            items_not_found += 1 if !update_id_dspace("items", lo)
          end
        elsif n_atts > bitstreams_not_found
          LearningObject::Attachment.where("id_dspace ~ '^[0-9]+$'").find_each do |att|
            bitstreams_not_found += 1 if !update_id_dspace("bitstreams", att)
          end
        else
          break
        end
      rescue PG::ConnectionBad, Faraday::TimeoutError, Net::HTTP::Persistent::Error, Net::ReadTimeout => e
        p "Database error, going to sleep"
        p e
        p e.class
        p e.message
        ActiveRecord::Base.clear_active_connections!
        # Sleeps for a while to wait database's recovery
        sleep(60.seconds)
        # Goes to next iteration to retry
        next
      end
    end
  end

  private

  def dspace_client
    @client ||= DspaceService.create_client
  end

  def get_col_items_from_dspace(limit, offset, expand = [])
    dspace_client.items.all(
      expand: expand.join(','),
      limit: limit,
      offset: offset
    )
  end

  def item_exists?(id)
    LearningObject.where(id_dspace: id).exists?
  end

  def update_id_dspace(obj_type, obj)
    item = dspace_client.send(obj_type).find(
      id: obj.id_dspace
    )
    old_dspace_id = obj.id_dspace
    obj.update_attributes(legacy_dspace_id: old_dspace_id, id_dspace: item.id)
    return true
  rescue Dspace::NotFoundError
    p "#{obj_type} id: #{obj.id_dspace} not found"
    return false
  end
end
