
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

require 'acceptance_helpers'
require 'shared/contexts'

resource 'Achievement' do
  header 'Accept', 'application/json'
  header 'Content-Type', 'application/json'

  explanation "Users can perform some pre-defined actions in the system to unlock achievements and win rewards."

  before { 12.times { create(:achievement) } }

  let(:achievements) { Achievement.all }

  get '/v1/achievements' do
    parameter :limit, 'Limit of achievements'
    parameter :offset, 'Offset of achievements'

    let(:limit) { 12 }
    let(:offset) { 0 }

    example_request 'Get a list of achievements' do
      expect(JSON.parse(response_body).map { |o| o['id'] }.sort).to eq(Achievement.limit(limit).offset(offset).pluck(:id).sort)
      expect(status).to eq(200)
    end
  end

  get '/v1/achievements/:id' do
    let(:id) { achievements.first.id }

    example 'Get an achievement' do
      do_request
      expect(path).to eq("/v1/achievements/#{id}") # `:id` is replaced with the value of `id`
      expect(response_body).to eq(Helper.serialize(Achievement.find(id)))
      expect(status).to eq(200)
    end
  end

  post '/v1/achievements' do
    include_context "authenticate_user_editor"

    parameter :name, 'The name of the achievement', scope: :achievement
    parameter :description, 'The description of the achievement', scope: :achievement
    parameter :reward_experience, 'Amount of experience rewarded when the achievement is completed', scope: :achievement
    parameter :reward_points, 'Amount of points rewarded when the achievement is completed', scope: :achievement
    parameter :state, 'State of the achievement [inactive, active, deleted]', scope: :achievement
    parameter :repeatable, 'Repeatability of the achievement [never, daily, weekly, monthly, yearly]', scope: :achievement
    parameter :resettable, 'Whether the achievement can be reseted or not', scope: :achievement
    parameter :requirements, 'Array with requirements ids', scope: :achievement

    let(:name) { Faker::Name.name }
    let(:description) { Faker::Lorem.paragraph}
    let(:reward_experience) { rand(0..100) }
    let(:reward_points) { rand(0..100) }
    let(:state) { "active" }
    let(:repeatable) { "daily" }
    let(:resettable) { true }
    let(:requirements) { [@requirement.id] }
    let(:raw_post) { params.to_json }

    before do
      @requirement = create(:requirement)
    end

    example 'Creating an achievement' do
      do_request
      expect(status).to eq(201)
    end
  end

  put '/v1/achievements/:id' do
    include_context "authenticate_user_editor"

    parameter :name, 'The name of the achievement', scope: :achievement
    parameter :description, 'The description of the achievement', scope: :achievement
    parameter :reward_experience, 'Amount of experience rewarded when the achievement is completed', scope: :achievement
    parameter :reward_points, 'Amount of points rewarded when the achievement is completed', scope: :achievement
    parameter :state, 'State of the achievement [inactive, active, deleted]', scope: :achievement
    parameter :repeatable, 'Repeatability of the achievement [never, daily, weekly, monthly, yearly]', scope: :achievement
    parameter :resettable, 'Whether the achievement can be reseted or not', scope: :achievement
    parameter :requirements, 'Array with requirements ids', scope: :achievement

    let(:id) { @achievement.id }
    let(:description) { Faker::Lorem.paragraph }
    let(:raw_post) { params.to_json }

    before do
      @achievement = create(:achievement)
    end

    example 'Updating an achievement' do
      do_request
      expect(status).to eq(200)
    end
  end

  delete '/v1/achievements/:id' do
    include_context "authenticate_user_editor"

    let(:id) { @achievement.id }

    before do
      @achievement = create(:achievement)
    end

    example 'Destroying an achievement' do
      do_request
      expect(JSON.parse(response_body)['state']).to eq("deleted")
      expect(status).to eq(200)
    end

  end

end
